/* --------------------------------------------------------------  */
/* (C)Copyright 2008                                               */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */

/*--------------------------------------------------------------------*/
/*   DaCS Fortran Bindings - c mapping implementation                 */
/*   The functions implemented here are called from Fortran code      */
/*   See dacsf_bindings.h for the external function prototypes        */
/*   and the specific implementation files.                           */
/*   Implementation files are grouped for spu space considerations.   */
/*--------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>
#include"dacs.h"
#include"dacsf_bindings.h"

/*--------------------------------------------------------------------*/
/*  DaCS Fortran API Wrappers.                                        */
/*  The C function names for the DaCS Fortran API wrappers follow     */
/*  the naming convention (dacs_init is used as an example):          */
/*                                                                    */
/*     Type                            Example wrapper name           */
/*     -------------------------       --------------------           */
/*     Function name                   dacsf_init_impl                */
/*     Upper Case Alias                DACSF_INIT                     */
/*     Lower Case Alias                dacsf_init                     */
/*     Singe Underscore Alias          dacsf_init_                    */
/*     Double Underscore Alias         dacsf_init__                   */
/*                                                                    */
/*  The macros FORTRAN_SUBR and FORTRAN_FUNC are used to generate     */
/*  the C function names and aliases for the Fortran bindings.        */
/*                                                                    */
/*  The Fortran compilers provide options to control the symbol name  */
/*  decoration:                                                       */
/*     Compiler  Options                                              */
/*     --------  -----------------------------------                  */
/*     gfortran   -fno-underscoring -fsecond-underscore               */
/*     xLF        -qextname -qnoextname -qmixed                       */
/*     PathScale  -fno-underscoring -fsecond-underscore               */
/*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*/
/* dacsf_error.c                                                      */
/*--------------------------------------------------------------------*/
FORTRAN_SUBR(DACSF_STRERROR,
             dacsf_strerror,(DACS_ERR_T *errcode_ptr,
                             char       *errstr,
                             int        errstr_len))
{
  const char *err = dacs_strerror(*errcode_ptr);
  memset(errstr,' ',errstr_len);
  if (err != NULL) {
     int msglen = strlen(err);
     memcpy(errstr,err,(msglen<errstr_len) ? msglen : errstr_len);
  }
}

FORTRAN_SUBR(DACSF_ERROR_NUM,
             dacsf_error_num,(int64_t      *error,
                              DACS_ERR_T   *errcode))
{
   *errcode = dacs_error_num(error);
}

FORTRAN_SUBR(DACSF_ERROR_CODE,
             dacsf_error_code,(int64_t      *error,
                               int32_t      *errcode,
                               DACS_ERR_T   *rc_ptr))
{
   *rc_ptr = dacs_error_code(error,(uint32_t*)errcode);
}

FORTRAN_SUBR(DACSF_ERRHANDLER_REG,
             dacsf_errhandler_reg,(void        *handler_ptr,
                                   int32_t     *flags_ptr,
                                   DACS_ERR_T  *rc_ptr))
{
   *rc_ptr = dacs_errhandler_reg((dacs_error_handler_t)(handler_ptr),*flags_ptr);
}
FORTRAN_SUBR(DACSF_ERROR_STR,
             dacsf_error_str,(int64_t      *error,
                              char         *errstr,
                              DACS_ERR_T   *rc_ptr,
                              int          errstr_len))
{
   const char *err;
   *rc_ptr = dacs_error_str(error,&err);
   if (*rc_ptr == DACS_SUCCESS) {
     memset(errstr,' ',errstr_len);
     int msglen = strlen(err);
     memcpy(errstr,err,(msglen<errstr_len) ? msglen : errstr_len);
   }
}

FORTRAN_SUBR(DACSF_ERROR_DE,
             dacsf_error_de,(int64_t      *error,
                             int32_t      *de,
                             DACS_ERR_T   *rc_ptr))
{
   *rc_ptr = dacs_error_de(error,(de_id_t*)de);
}

FORTRAN_SUBR(DACSF_ERROR_PID,
             dacsf_error_pid,(int64_t      *error,
                              int64_t      *pid,
                              DACS_ERR_T   *rc_ptr))
{
   *rc_ptr = dacs_error_pid(error,(dacs_process_id_t*)pid);
}

