/* --------------------------------------------------------------  */
/* (C)Copyright 2008                                               */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */

/*--------------------------------------------------------------------*/
/*   DaCS Fortran Bindings - c mapping implementation                 */
/*   The functions implemented here are called from Fortran code      */
/*   See dacsf_bindings.h for the external function prototypes        */
/*   and the specific implementation files.                           */
/*   Implementation files are grouped for spu space considerations.   */
/*--------------------------------------------------------------------*/
#include"dacs.h"
#include"dacsf_bindings.h"

/*--------------------------------------------------------------------*/
/*  DaCS Fortran API Wrappers.                                        */
/*  The C function names for the DaCS Fortran API wrappers follow     */
/*  the naming convention (dacs_init is used as an example):          */
/*                                                                    */
/*     Type                            Example wrapper name           */
/*     -------------------------       --------------------           */
/*     Function name                   dacsf_init_impl                */
/*     Upper Case Alias                DACSF_INIT                     */
/*     Lower Case Alias                dacsf_init                     */
/*     Singe Underscore Alias          dacsf_init_                    */
/*     Double Underscore Alias         dacsf_init__                   */
/*                                                                    */
/*  The macros FORTRAN_SUBR and FORTRAN_FUNC are used to generate     */
/*  the C function names and aliases for the Fortran bindings.        */
/*                                                                    */
/*  The Fortran compilers provide options to control the symbol name  */
/*  decoration:                                                       */
/*     Compiler  Options                                              */
/*     --------  -----------------------------------                  */
/*     gfortran   -fno-underscoring -fsecond-underscore               */
/*     xLF        -qextname -qnoextname -qmixed                       */
/*     PathScale  -fno-underscoring -fsecond-underscore               */
/*--------------------------------------------------------------------*/

/*--------------------------------------------------------------------*/
/* dacsf_send_recv.c                                                  */
/*--------------------------------------------------------------------*/
FORTRAN_SUBR(DACSF_RECV,
             dacsf_recv,(pvoid_holder     dst_data,
                         int32_t          *len_ptr,
                         int32_t          *src_de_ptr,
                         int64_t          *src_pid_ptr,
                         int32_t          *stream_ptr,
                         int32_t          *wid_ptr,
                         DACS_BYTE_SWAP_T *swap_ptr,
                         DACS_ERR_T       *rc_ptr))
{
  #ifdef DACS_ERROR_CHECKING
    if ((*len_ptr<0)) {
      *rc_ptr = DACS_ERR_INVALID_SIZE;
      return;
    }
  #endif
  *rc_ptr = dacs_recv(dacsf_makeptr_impl(dst_data),
                      (uint32_t)*len_ptr,
                      (de_id_t)*src_de_ptr,
                      (dacs_process_id_t)*src_pid_ptr,
                      (uint32_t)*stream_ptr,
                      (dacs_wid_t)*wid_ptr,
                      *swap_ptr);
}

FORTRAN_SUBR(DACSF_SEND,
             dacsf_send,(pvoid_holder     src_data,
                         int32_t          *len_ptr,
                         int32_t          *dst_de_ptr,
                         int64_t          *dst_pid_ptr,
                         int32_t          *stream_ptr,
                         int32_t          *wid_ptr,
                         DACS_BYTE_SWAP_T *swap_ptr,
                         DACS_ERR_T       *rc_ptr))
{
  #ifdef DACS_ERROR_CHECKING
    if ((*len_ptr<0)) {
      *rc_ptr = DACS_ERR_INVALID_SIZE;
      return;
    }
  #endif
  *rc_ptr = dacs_send(dacsf_makeptr_impl(src_data),
                      (uint32_t)*len_ptr,
                      (de_id_t)*dst_de_ptr,
                      (dacs_process_id_t)*dst_pid_ptr,
                      (uint32_t)*stream_ptr,
                      (dacs_wid_t)*wid_ptr,
                      *swap_ptr);
}


